DVCLIVE_LOGLEVEL = "DVCLIVE_LOGLEVEL"
DVCLIVE_OPEN = "DVCLIVE_OPEN"
DVCLIVE_RESUME = "DVCLIVE_RESUME"
DVC_CHECKPOINT = "DVC_CHECKPOINT"
DVC_EXP_BASELINE_REV = "DVC_EXP_BASELINE_REV"
DVC_EXP_NAME = "DVC_EXP_NAME"
